SHARED CONST PCM_fifo = $9F3D
SHARED CONST AUDIO_CTRL = $9F3B
SHARED CONST AUDIO_RATE = $9F3C
SHARED CONST STEREO_ON  = %00010000
SHARED CONST STEREO_OFF = %11101111
SHARED CONST BIT16_ON = %00100000
SHARED CONST FIFO_EMPTY = %01000000
SHARED CONST FIFO_FULL = %10000000
SHARED CONST AFLOW_ON = %00001000

DIM loop_ctr1 AS BYTE FAST
DIM SHARED CTRL AS BYTE @$9F3B
DIM SHARED FIFO AS BYTE @$9F3D
DIM SHARED VERA_IEN AS BYTE @$9F26
DIM SHARED VERA_ISR AS BYTE @$9F27
DIM SHARED PlayDone AS BYTE
DIM SHARED PCM_CFG_VAR AS BYTE
DIM SHARED PCM_VOLUME AS BYTE
DIM SHARED PCM_V_RATE AS BYTE
DIM SHARED IsStereo AS BYTE
DIM SHARED IsSupported AS BYTE
DIM SHARED SampleRate AS LONG
DIM SHARED AUDIO_hz AS FLOAT
DIM SHARED Byte_Rate AS FLOAT
DIM SHARED PCM_LFN AS BYTE FAST
DIM SHARED PCM_ERROR AS BYTE
DIM SHARED AUDIO_FILE AS STRING * 96
DIM SHARED PCM_MSG AS STRING*40

DIM FSCNT AS BYTE FAST
DIM RIFFHDR AS STRING * 44 @$A000


FUNCTION FIFO_LOW AS BYTE() STATIC SHARED
 RETURN (VERA_ISR AND AFLOW_ON) = AFLOW_ON
END FUNCTION

FUNCTION FIFO_Is_FULL AS BYTE() STATIC SHARED
 RETURN (CTRL AND FIFO_FULL) = FIFO_FULL
END FUNCTION

FUNCTION FIFO_Is_Empty AS BYTE() STATIC SHARED
 RETURN (CTRL AND FIFO_EMPTY) = FIFO_EMPTY
END FUNCTION

FUNCTION CALC_VERA_RATE AS LONG(TRUERATE AS LONG) STATIC
 RETURN TRUERATE / CLONG(380)
END FUNCTION

SUB PausePCMStream() STATIC SHARED
  POKE AUDIO_RATE, 0
kloop:
  GET _Key
  IF _Key<> 32 AND _Key<>27 THEN GOTO kloop
  POKE AUDIO_RATE, PCM_V_RATE
END SUB

SUB PCM_SetVolume() STATIC SHARED
 DIM TMPB AS BYTE
 TMPB = PCM_CFG_VAR AND %11110000
 PCM_CFG_VAR = TMPB OR PCM_VOLUME
 POKE AUDIO_CTRL, PCM_CFG_VAR
END SUB

SUB VolumeDown() STATIC SHARED
 IF PCM_VOLUME > 1 THEN
    PCM_VOLUME=PCM_VOLUME-1
    CALL PCM_SetVolume()
 END IF
END SUB

SUB VolumeUp() STATIC SHARED
 IF PCM_VOLUME < 15 Then
    PCM_VOLUME = PCM_VOLUME+1
    CALL PCM_SetVolume()
 END IF
END SUB

SUB FILL_AUDIO_FIFO() static
    ASM
      ldx {PCM_LFN}
      jsr {CHKIN}
      ldx #17
      sta {loop_ctr1}
loop  ldx #$3D
      ldy #$9f
      lda #240
      sec
      jsr {MACPTR}
      jsr {READST}
      bne done
      dec {loop_ctr1}
      bne loop
      clc
      bcc fend
done  lda $FF
      sta {PlayDone}
fend  jsr {CLRCHN}
    END ASM
END SUB

SUB TWEAK_AUDIO_FIFO() static
    ASM
      ldx {PCM_LFN}
      jsr {CHKIN}
      ldx {FSCNT}
      stx {loop_ctr1}
loop2 ldx #$3D
      ldy #$9f
      lda #0
      sec
      jsr {MACPTR}
      jsr {READST}
      bne done2
      dec {loop_ctr1}
      bne loop2
      jmp fend2
done2 lda $FF
      sta {PlayDone}
fend2 jsr {CLRCHN}
    END ASM
END SUB

SUB CalculateRates() STATIC
  AUDIO_hz=(PCM_V_RATE/128.0) * 48828.15
  Byte_Rate = AUDIO_hz
  IF (PCM_CFG_VAR AND BIT16_ON) = BIT16_ON THEN Byte_Rate = Byte_Rate*2.0
  IF (PCM_CFG_VAR AND STEREO_ON) = STEREO_ON THEN Byte_Rate = Byte_Rate*2.0
  Byte_Rate = CLONG(Byte_Rate)
END SUB

SUB OpenZCM(ZCMFILE AS STRING*96) STATIC SHARED
   OPEN PCM_LFN,8,PCM_LFN,ZCMFILE+",s,r"
   ASM
    lda {SCRATCHBANK}
    sta 0
    ldx {PCM_LFN}
    jsr {CHKIN}
    ldx #$01
    ldy #$A0
    lda #8
    sta {RIFFHDR}
    jsr {MACPTR}
    jsr {CLRCHN}
  END ASM
  PCM_V_RATE = ASC(MID$(RIFFHDR,7,1))
  PCM_CFG_VAR = ASC(MID$(RIFFHDR,6,1))
  CALL CalculateRates()
  PCM_CFG_VAR = PCM_CFG_VAR AND %11110000
  PCM_CFG_VAR = PCM_CFG_VAR OR PCM_VOLUME
END SUB

FUNCTION Open_WAVE AS BYTE(WAVFILE AS STRING*96) STATIC SHARED
 DIM IsWave AS BYTE
 DIM IsPCM AS BYTE
 DIM PCMBits AS BYTE
 DIM NumChannels AS BYTE
 DIM TMP1 AS WORD
 DIM TMP2 AS WORD
 DIM TMP3 AS LONG
 DIM TMP4 AS WORD
   PCM_ERROR=FALSE
   OPEN PCM_LFN,8,PCM_LFN,WAVFILE+",s,r"
   ASM
    lda {SCRATCHBANK}
    sta 0
    lda #44
    sta {RIFFHDR}
    ldx {PCM_LFN}
    jsr {CHKIN}
    ldx #$01
    ldy #$A0
    lda #44
    jsr {MACPTR}
    jsr {CLRCHN}
  END ASM
  IsWave = (LEFT$(RIFFHDR,4) = "riff") AND (MID$(RIFFHDR,8,4)="wave")
  PCMBits = ASC(MID$(RIFFHDR,34,1))
  TMP1 = ASC(MID$(RIFFHDR,20,1))
  TMP2 = ASC(MID$(RIFFHDR,21,1))
  TMP4 = CWORD(TMP1) + CWORD(256*TMP2)
  IsPCM = (TMP4=1)
  TMP1 = ASC(MID$(RIFFHDR,24,1))
  TMP2 = CWORD(ASC(MID$(RIFFHDR,25,1))) * 256
  TMP3 = CLONG(ASC(MID$(RIFFHDR,26,1)) * 65536)
  SampleRate = TMP1+TMP2+TMP3
  NumChannels = ASC(MID$(RIFFHDR,22,1))
  IsStereo = (NumChannels=2)
  IsSupported = (IsWave AND (PCMBits=16)) AND IsPCM AND ((NumChannels > 0) AND (NumChannels < 3))

  IF IsSupported THEN
     PCM_V_RATE=CBYTE(CALC_VERA_RATE(SampleRate))
     PCM_CFG_VAR = BIT16_ON
     IF IsStereo THEN PCM_CFG_VAR = (PCM_CFG_VAR OR STEREO_ON)
     PCM_CFG_VAR = (PCM_CFG_VAR OR PCM_VOLUME)
     CALL CalculateRates()
  END IF

  IF NOT IsSupported THEN CLOSE PCM_LFN:PCM_ERROR=TRUE
  RETURN IsSupported
END FUNCTION

SUB PlayPCMStream(Control_Callback AS WORD,ScreenUpdate AS Word) STATIC SHARED
   _Key = 0
   PlayDone = 0
   IF Is_Host_FS THEN FSCNT = 12 ELSE FSCNT=6
   IF Byte_Rate > 80000.0 THEN FSCNT = FSCNT + 1
   IF Byte_Rate > 100000.0 THEN FSCNT = FSCNT + 1
   POKE AUDIO_RATE, 0
   POKE AUDIO_CTRL, PCM_CFG_VAR
   CALL FILL_AUDIO_FIFO()
   CALL ZeroTimer()
   POKE AUDIO_RATE, PCM_V_RATE
 DO
  IF (VERA_ISR AND AFLOW_ON) = AFLOW_ON THEN
     CALL TWEAK_AUDIO_FIFO()
        ASM
         lda {ScreenUpdate}+1
         beq overscrn
         jsr scrnproc
         jmp overscrn+1
scrnproc jmp ({ScreenUpdate})
overscrn nop
       END ASM
   END IF

       ASM
        lda {Control_Callback}+1
        beq over
        jsr callit
        jmp over+1
callit  jmp ({Control_Callback})
over    nop
       END ASM


   IF _Key > 0 THEN
      IF _Key = 27 THEN PlayDone = TRUE
      IF _Key = 32 THEN Call PausePCMStream()
      IF _Key = 145 THEN Call VolumeUp()
      IF _Key = 17 THEN Call VolumeDown()
   END IF
 LOOP UNTIL PlayDone
END SUB

SUB Play_WAV(WAVNAME AS STRING * 80,pVOL AS BYTE) STATIC SHARED
DIM Save_VOLUME AS BYTE
   Save_VOLUME = PCM_VOLUME
   IF pVOL > 15 THEN pVOL=15
   IF Open_WAVE(ASCII_UPPER(WAVNAME)) THEN
      PCM_VOLUME = pVOL
      CALL PlayPCMStream(0,0)
   END IF
   PCM_VOLUME = Save_VOLUME
END SUB


SUB PCM_INIT() STATIC SHARED
  PCM_LFN = 6
  SCRATCHBANK = 2
  CALL TEST_FILESYSTEM(FALSE)
  _Key = 0
  PCM_ERROR = 0
  PCM_VOLUME = 12
END SUB

